<?php

namespace Drupal\Tests\field_location\Functional;

use Drupal\Core\Entity\Element\EntityAutocomplete;
use Drupal\googlemaps\Entity\GoogleMap;
use Drupal\Tests\field_ui\Traits\FieldUiTestTrait;
use Drupal\Tests\BrowserTestBase;

/**
 * Tests the location field.
 *
 * @group field_location
 */
class ManageDisplayTest extends BrowserTestBase {

  use FieldUiTestTrait;

  protected const FIELD_TYPE = 'location';
  protected const FIELD_NAME = 'field_location_test';

  /**
   * {@inheritdoc}
   */
  protected static $modules = [
    // Fields will be managed using dummy content type of "node" entity.
    'node',
    // Needed to display block with breadcrumbs.
    'block',
    // Required by "FieldUiTestTrait".
    'field_ui',
    // Contains schemes definitions for the "location" field type.
    'field_location_test',
  ];

  /**
   * {@inheritdoc}
   */
  protected $defaultTheme = 'classy';

  /**
   * The content type for testing.
   *
   * @var \Drupal\node\NodeTypeInterface
   */
  protected $contentType;

  /**
   * {@inheritdoc}
   */
  protected function setUp(): void {
    parent::setUp();

    $this->drupalPlaceBlock('system_breadcrumb_block');

    $this->drupalLogin($this->drupalCreateUser([
      'access content',
      'bypass node access',
      'administer node fields',
      'administer node display',
      'administer node form display',
      'administer content types',
    ]));

    $this->contentType = $this->drupalCreateContentType([
      'type' => $this->randomMachineName(),
    ]);
  }

  /**
   * Tests field creation.
   */
  public function testCreateField(): void {
    $this->fieldUIAddNewField('admin/structure/types/manage/' . $this->contentType->id(), static::FIELD_NAME, NULL, static::FIELD_TYPE, [
      'settings[googlemaps_library]' => EntityAutocomplete::getEntityLabels(
        $this->container->get('entity_type.manager')->getStorage(GoogleMap::ENTITY_TYPE)->loadMultiple(['field_location'])
      ),
    ]);
    // @todo Continue here.
  }

}
